@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,7 +9,6 @@
 
 #include <util/buffer.h>
 
-#include <boost/function.hpp>
 #include <boost/shared_ptr.hpp>
 
 #include <map>
@@ -55,6 +54,22 @@ public:
         isc::Exception(file, line, what) { };
 };
 
+/// @brief Exception thrown during option unpacking
+/// This exception is thrown when an error has occurred unpacking
+/// an option from a packet and rather than drop the whole packet, we
+/// wish to simply skip over the option (i.e. omit it from the unpacked
+/// results), and resume unpacking with the next option in the buffer.
+/// The intent is to allow us to be liberal with what we receive, and
+/// skip nonsensical options rather than drop the whole packet. This
+/// exception is thrown, for instance, when string options are found to
+/// be empty or to contain only nuls.
+class SkipThisOptionError : public Exception {
+public:
+    SkipThisOptionError (const char* file, size_t line, const char* what) :
+        isc::Exception(file, line, what) { };
+};
+
+
 class Option {
 public:
     /// length of the usual DHCPv4 option header (there are exceptions)
@@ -165,6 +180,33 @@ public:
     /// @param source Option to be copied.
     Option(const Option& source);
 
+    /// @brief Factory function creating an instance of the @c Option.
+    ///
+    /// This function should be used to create an instance of the DHCP
+    /// option within a hooks library in cases when the library may be
+    /// unloaded before the object is destroyed. This ensures that the
+    /// ownership of the object by the Kea process is retained.
+    ///
+    /// @param u specifies universe (V4 or V6)
+    /// @param type option type (0-255 for V4 and 0-65535 for V6)
+    ///
+    /// @return Pointer to the @c Option instance.
+    static OptionPtr create(Universe u, uint16_t type);
+
+    /// @brief Factory function creating an instance of the @c Option.
+    ///
+    /// This function should be used to create an instance of the DHCP
+    /// option within a hooks library in cases when the library may be
+    /// unloaded before the object is destroyed. This ensures that the
+    /// ownership of the object by the Kea process is retained.
+    ///
+    /// @param u specifies universe (V4 or V6)
+    /// @param type option type (0-255 for V4 and 0-65535 for V6)
+    /// @param data content of the option
+    ///
+    /// @return Pointer to the @c Option instance.
+    static OptionPtr create(Universe u, uint16_t type, const OptionBuffer& data);
+
     /// @brief Assignment operator.
     ///
     /// The assignment operator performs a deep copy of the option and
@@ -332,21 +374,21 @@ public:
     /// @return uint32_t value stored on first four bytes
     uint32_t getUint32() const;
 
-    /// @brief Sets content of this option to singe uint8 value.
+    /// @brief Sets content of this option to a single uint8 value.
     ///
     /// Option it resized appropriately (to length of 1 octet).
     ///
     /// @param value value to be set
     void setUint8(uint8_t value);
 
-    /// @brief Sets content of this option to singe uint16 value.
+    /// @brief Sets content of this option to a single uint16 value.
     ///
     /// Option it resized appropriately (to length of 2 octets).
     ///
     /// @param value value to be set
     void setUint16(uint16_t value);
 
-    /// @brief Sets content of this option to singe uint32 value.
+    /// @brief Sets content of this option to a single uint32 value.
     ///
     /// Option it resized appropriately (to length of 4 octets).
     ///
